//
//	Author					: CYTRON	
//	Project					: SK40C + SKPS
//	Project description		: COntrolling SK40C Using SKPS
//							  This sample source code is valid for 20MHz Crystal
//
//=======================================================================



//	include
//=======================================================================
#include <pic.h> 

//	configuration
//=======================================================================
__CONFIG ( 0x3F32 );

//	define
//=======================================================================
#define	rs			RB4				//RS pin of the LCD display
#define	e			RB5				//E pin of the LCD display

#define	lcd_data	PORTD			//LCD 8-bit data PORT

#define	SW1			RB0			
#define	SW2			RB1			

#define	LED1		RB6			
#define	LED2		RB7			

//skps protocol
#define	p_select		0
#define p_joyl			1
#define p_joyr			2
#define p_start			3
#define p_up			4
#define p_right			5
#define p_down			6
#define p_left			7
#define	p_l2			8
#define	p_r2			9
#define p_l1			10
#define p_r1			11
#define p_triangle		12
#define p_circle		13
#define p_cross			14
#define	p_square		15
#define p_joy_lx		16
#define	p_joy_ly		17
#define p_joy_rx		18
#define p_joy_ry		19
#define p_joy_lu		20
#define p_joy_ld		21
#define p_joy_ll		22
#define p_joy_lr		23
#define p_joy_ru		24
#define p_joy_rd		25
#define p_joy_rl		26
#define p_joy_rr		27

#define	p_con_status	28
#define p_motor1		29
#define p_motor2		30

//	global variable
//=======================================================================



//	function prototype
//=======================================================================
void init(void);
void delay(unsigned long data);
void send_config(unsigned char data);
void send_char(unsigned char data);
void e_pulse(void);
void lcd_goto(unsigned char data);
void lcd_clr(void);
void send_string(const char *s);

void uart_send(unsigned char data);
unsigned char uart_rec(void);
unsigned char skps(unsigned char data);
void skps_vibrate(unsigned char motor, unsigned char value);

//	interrupt prototype
//=======================================================================
static void interrupt isr(void)
{

} 

//	main function
//=======================================================================
void main(void)
{

	init();									//initialize microcontroller
	
	lcd_clr();								// clear LCD screen 				
	lcd_goto(0);							// set the position of message to be displayed
	send_string("Controlling SK40C");		// send message to be displayed 
	lcd_goto(20);							// set the position of message to be displayed
	send_string("   Using SKPS");			// send message to be displayed 
	
	while(1)
	{	
		if(skps(p_l1)==0)LED1=1;			 // if button L1 pressed, on LED1
		else LED1=0;						
		
		if(skps(p_r1)==0)LED2=1;			 // if button R1 pressed, on LED2
		else LED2=0;
		
		if(skps(p_up)==0)					 // check "up" button
		{
			lcd_goto(20);					 // set the position of message to be displayed
			send_string("     UP      ");	 // Display button pressed on LCD.
		}
		else if(skps(p_down)==0)			 // check "down" button
		{	
			lcd_goto(20);					 // set the position of message to be displayed
			send_string("    DOWN      ");	 // Display button pressed on LCD.
		}
		else if(skps(p_left)==0)			 // check "left" button
		{
			lcd_goto(20);					 // set the position of message to be displayed
			send_string("    LEFT      ");   // Display button pressed on LCD.
		}
		else if(skps(p_right)==0)			 //check "right" button
		{
			lcd_goto(20);					 // set the position of message to be displayed
			send_string("    RIGHT      ");  // Display button pressed on LCD.
		}
	
	}
}

// initailization
//=======================================================================
void init()
{
		//set I/O input output
	TRISB = 0b00000011;					//configure PORTB I/O direction
	TRISD = 0b00000000;					//configure PORTD I/O direction
	TRISA = 0b00000111;					//configure PORTA I/O direction

	//Configure UART
	SPBRG=0x81;			//set baud rate as 9600 baud
	BRGH=1;				//baud rate high speed option
	TXEN=1;				//enable transmission
	TX9 =0;				//8-bit transmission
	RX9 =0;				//8-bit reception	
	CREN=1;				//enable reception
	SPEN=1;				//enable serial port

	//setup ADC
	ADCON1 = 0b00000110;				//set ADx pin digital I/O
	
	//configure lcd
	send_config(0b00000001);			//clear display at lcd
	send_config(0b00000010);			//lcd return to home 
	send_config(0b00000110);			//entry mode-cursor increase 1
	send_config(0b00001100);			//display on, cursor off and cursor blink off
	send_config(0b00111000);			//function set

	//display startup message	
	lcd_clr();							//clear lcd
	lcd_goto(2);						//set the lcd cursor to location 0
	send_string("Cytron Tech.");		//display "Cytron Tech."
	lcd_goto(26);						//set the lcd cursor to location 20
	send_string("SK40C");				//display "PR18"

	LED1=0;
	LED2=0;
}

// uart function
//=======================================================================
void uart_send(unsigned char data)	//function to send out a byte via uart
{	
	while(TXIF==0);					//wait for previous data to finish send out
	TXREG=data;						//send new data
}

unsigned char uart_rec(void)		//function to wait for a byte receive from uart
{
	unsigned char temp;
	while(RCIF==0);					//wait for data to received
	temp=RCREG;					
	return temp;					//return the received data
}

// skps function
//=======================================================================
unsigned char skps(unsigned char data)			//function to read button and joystick 
{												//information on ps controller
	uart_send(data);
	return uart_rec();
}

void skps_vibrate(unsigned char motor, unsigned char value)
{												//function to control the vibrator motor 
	uart_send(motor);							//on ps controller
	uart_send(value);	
}


//	LCD	functions
//=======================================================================
void delay(unsigned long data)
{
	for( ;data>0;data-=1);
}

void send_config(unsigned char data)
{
	rs=0;										//clear rs into config mode 
	lcd_data=data;
	delay(300);
	e_pulse();
}

void send_char(unsigned char data)
{
	rs=1;										//set rs into write mode
	lcd_data=data;						 
	delay(300);
	e_pulse();
}

void e_pulse(void)
{
	e=1;
	delay(300);
	e=0;
	delay(300);
}

void lcd_goto(unsigned char data)
{
 	if(data<16)
	{
	 	send_config(0x80+data);
	}
	else
	{
	 	data=data-20;
		send_config(0xc0+data);
	}
}

void lcd_clr(void)
{
 	send_config(0x01);
	delay(350);	
}

void send_string(const char *s)
{          
	unsigned char i=0;
  	while (s && *s)send_char (*s++);

}


